
/*
 *  Copyright (c) 2021 Microsoft Corporation
 *
 *  This program and the accompanying materials are made available under the
 *  terms of the Apache License, Version 2.0 which is available at
 *  https://www.apache.org/licenses/LICENSE-2.0
 *
 *  SPDX-License-Identifier: Apache-2.0
 *
 */
package org.fraunhofer.fit.extension.extractor;

import jakarta.ws.rs.Consumes;
import jakarta.ws.rs.GET;
import jakarta.ws.rs.Path;
import jakarta.ws.rs.Produces;
import jakarta.ws.rs.QueryParam;
import jakarta.ws.rs.core.MediaType;
import jakarta.ws.rs.core.Response;
import org.eclipse.edc.spi.monitor.Monitor;


@Consumes({MediaType.APPLICATION_JSON})
@Produces({MediaType.APPLICATION_JSON})
@Path("/")
public class ExtractorApiController {

    private final Monitor monitor;

    public ExtractorApiController(Monitor monitor) {
        this.monitor = monitor;
    }


    @GET
    @Path("get-fairness-score")
    public Response getFairnessScore(
            @QueryParam("assetPath") String assetPath,
            @QueryParam("assetpid") String assetPid,
            @QueryParam("assetDataLink") String assetDataLink,
            @QueryParam("assetVersion") String assetVersion
    ) {
        // Check if the assetPath is provided
        if (assetPath == null || assetPath.isEmpty()) {
            // Return an error response if assetPath is missing
            return Response.status(Response.Status.BAD_REQUEST)
                    .entity("{\"error\":\"assetPath is required.\"}")
                    .type(MediaType.APPLICATION_JSON)
                    .build();
        }
        if (assetPid != null) {
            assetPid = "";
        }
        if (assetDataLink != null) {
            assetDataLink = "";
        }
        if (assetVersion != null) {
            assetVersion = "";
        }

        // Create an object of the class responsible for metadata extraction
        MetadataExtractor metadataExtractor = new MetadataExtractor(monitor);

        // Call the method to get the fair score and metadata
        var fairnessProperties = metadataExtractor.getFairScore(
                assetPath,
                assetPid,
                assetDataLink,
                assetVersion
        );

        // Return the metadata as a JSON response
        return Response.ok(fairnessProperties)
                .type(MediaType.APPLICATION_JSON)
                .build();
    }


}

